import { select } from 'canknow-core';
import Message from './../components/message';
import { t } from '../locale/index';

export default function ({ text, showSuccessTip, showFailureTip, successCallback, failureCallback }) {
    const container = document.body;
    const isRTL = document.documentElement.getAttribute('dir') === 'rtl';
    const fakeElem = document.createElement('textarea');
    showSuccessTip = showSuccessTip || t('core.copy.success');
    showFailureTip = showFailureTip || t('core.copy.fail');
    fakeElem.style.fontSize = '12pt';
    fakeElem.style.border = '0';
    fakeElem.style.padding = '0';
    fakeElem.style.margin = '0';
    fakeElem.style.position = 'absolute';
    fakeElem.style[isRTL ? 'right' : 'left'] = '-9999px';
    let yPosition = window.pageYOffset || document.documentElement.scrollTop;
    fakeElem.style.top = `${yPosition}px`;
    fakeElem.setAttribute('readonly', '');
    fakeElem.value = text;
    container.appendChild(fakeElem);
    select(fakeElem);
    try {
        document.execCommand('copy');
        if (showSuccessTip) {
            Message.success(showSuccessTip);
        }
        if (successCallback) {
            successCallback.call();
        }
    }
    catch (err) {
        if (showFailureTip) {
            Message.error(showFailureTip);
        }
        if (failureCallback) {
            failureCallback.call();
        }
    }
}
